import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:jm_basic_flutter/extends/extensions.dart';
import 'always_keep_cursor_end_text_editing_controller.dart';

class QuantityInputField extends StatefulWidget {
  const QuantityInputField(
      {required this.quantity,
      this.width = 140,
      this.height = 32,
      this.minus,
      this.add,
      this.onQuantityChanged,
      this.minimumQuantity = 1,
      this.maximumQuantity = 9999,
      this.whenBeyondMaximumQuantity});

  final int quantity;
  final double width;
  final double height;
  final IconData? minus;
  final IconData? add;
  final int minimumQuantity;
  final int maximumQuantity;
  final Function(int)? onQuantityChanged;
  final VoidCallback? whenBeyondMaximumQuantity;

  @override
  _QuantityInputFieldState createState() => _QuantityInputFieldState();
}

class _QuantityInputFieldState extends State<QuantityInputField> {
  int? quantity;
  AlwaysKeepCursorEndTextEditingController? textEditingController;
  FocusNode focusNode = FocusNode();

  bool get isMinusButtonEnabled => quantity! > widget.minimumQuantity;

  bool get isPlusButtonEnabled => quantity! < widget.maximumQuantity;

  @override
  initState() {
    super.initState();
    this.quantity = widget.quantity;
    if (this.quantity == null || this.quantity! < widget.minimumQuantity) {
      this.quantity = widget.minimumQuantity;
    }
    this.textEditingController = AlwaysKeepCursorEndTextEditingController(text: '${this.quantity}');
    this.focusNode.addListener(() {
      if (!this.focusNode.hasFocus) {
        _processQuantityBounding();
      }
    });
  }

  Widget _makeButton({IconData? icon, required bool isEnabled, required BorderRadius borderRadius, VoidCallback? onPressed}) {
    return RawMaterialButton(
      onPressed: isEnabled ? onPressed : null,
      padding: EdgeInsets.zero,
      shape: RoundedRectangleBorder(borderRadius: borderRadius),
      elevation: 0,
      materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
      constraints: BoxConstraints(minWidth: 32, maxWidth: 32),
      fillColor: Color(0xFFF3F4F7),
      child: Center(
        child: Icon(
          icon,
          size: 16,
          color: isEnabled ? Color(0xFF777777) : Color(0xFFCCCCCC),
        ),
      ),
    );
  }

  bool quantityIsSafe(int qty) {
    return qty >= widget.minimumQuantity && qty <= widget.maximumQuantity;
  }

  getSafeQuantity(int qty) {
    if (qty < widget.minimumQuantity) {
      return widget.minimumQuantity;
    } else if (qty > widget.maximumQuantity) {
      return widget.maximumQuantity;
    }
    return qty;
  }

  /// 输入完成后，处理数量边界
  _processQuantityBounding() {
    int quantity = widget.minimumQuantity;
    try {
      quantity = int.parse(textEditingController!.text);
    } catch (e) {} finally {
      quantity = getSafeQuantity(quantity);
      setState(() {
        this.quantity = quantity;
      });
    }
    textEditingController!.text = '${this.quantity}';
    if (widget.onQuantityChanged != null) {
      widget.onQuantityChanged!(quantity);
    }
  }

  /// 输入回调
  _onInputChanged(String text) {
    int? quantity;
    try {
      quantity = int.parse(text);
    } catch (e) {} finally {
      if (quantity != null && quantityIsSafe(quantity)) {
        this.quantity = quantity;
        if (widget.onQuantityChanged != null) {
          widget.onQuantityChanged!(quantity);
        }
      }
    }
  }

  /// 点击减号
  _onMinus() {
    if (quantity! > widget.minimumQuantity) {
      _updateQuantity(quantity! - 1);
    }
  }

  /// 点击加号
  _onPlus() {
    if (quantity! < widget.maximumQuantity) {
      _updateQuantity(quantity! + 1);
    } else {
      if (widget.whenBeyondMaximumQuantity != null) {
        widget.whenBeyondMaximumQuantity!();
      }
    }
  }

  _updateQuantity(int quantity) {
    // focusNode.unfocus();
    setState(() {
      this.quantity = quantity;
    });
    String quantityText = '$quantity';
    textEditingController!.text = quantityText;
    if (widget.onQuantityChanged != null) {
      widget.onQuantityChanged!(quantity);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: widget.width,
      height: widget.height,
      decoration: BoxDecoration(color: Colors.white, border: Border.all(color: Color(0xFFCCCCCC)), borderRadius: BorderRadius.circular(4)),
      child: Row(
        children: [
          _makeButton(
              icon: widget.minus ?? Icons.minimize, isEnabled: isMinusButtonEnabled, borderRadius: BorderRadius.horizontal(left: Radius.circular(4)), onPressed: _onMinus),
          Expanded(
              child: TextField(
            controller: textEditingController,
            textAlign: TextAlign.center,
            focusNode: focusNode,
            onTap: () => textEditingController!.selectAll(),
            enableInteractiveSelection: false,
            inputFormatters: [FilteringTextInputFormatter.digitsOnly],
            onChanged: _onInputChanged,
            maxLength: 4,
            keyboardType: TextInputType.number,
            decoration: InputDecoration(border: InputBorder.none, isDense: true, counterText: ''),
          )),
          _makeButton(icon: widget.add ?? Icons.add, isEnabled: isPlusButtonEnabled, borderRadius: BorderRadius.horizontal(right: Radius.circular(4)), onPressed: _onPlus)
        ],
        crossAxisAlignment: CrossAxisAlignment.stretch,
      ),
    );
  }
}
